##############################################################################
#
# Copyright (c) 2002 Ingeniweb SARL
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""
$Id: ExFile.py,v 1.31 2005/09/12 15:48:50 pjgrizel Exp $
"""

__author__ = ''
__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo
from ZODB.POSException import ConflictError

# CMF imports
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from ComputedAttribute import ComputedAttribute

# Archetypes imports
from Products.Archetypes.utils import fixSchema
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

# PloneExFile imports
from Products.AttachmentField.AttachmentField import AttachmentField
from Products.AttachmentField.AttachmentWidget import AttachmentWidget
from Products.PloneExFile.config import PROJECTNAME
from Products.PloneExFile.ATContentType import FileContent
from Products.PloneExFile import Permissions
from Products.PloneExFile.ExFileLock import ExFileLock

PloneExFileSchema = FileContent.schema + \
        ExFileLock.schema + Schema((
        AttachmentField(
            'file',
            primary=True,
            searchable=True,
            #languageIndependent=True,
            widget = AttachmentWidget(
                description = "Select the file to be added by clicking the 'Browse' button.",
                description_msgid = "help_file",
                label= "File",
                label_msgid = "label_file",
                i18n_domain = "plone",
                show_content_type = False,
                ),
            ),
        ), marshall=PrimaryFieldMarshaller())

class PloneExFile(ExFileLock, FileContent):
    """PloneExFile class"""
    
    content_icon   = 'ExFile.gif'
    meta_type      = 'ExFile'
    archetype_name = 'File'
    default_view = 'ExFile_view'
    immediate_view = 'ExFile_view'
    suppl_views    = ()
    newTypeFor     = ()
    typeDescription= ''
    typeDescMsgId  = ''
    assocMimetypes = ()
    assocFileExt   = ()
    
    use_folder_tabs = 0
    filter_content_types  = 0
    allowed_content_types = ()
    include_default_actions = 0
    schema = PloneExFileSchema
    
    actions = (
         {
          'id'          : 'view',
          'name'        : 'View',
          'action'      : 'string:${object_url}/view',
          'permissions' : (CMFCorePermissions.View,),
          'visible'     : 1,
         },
         {
          'id'          : 'edit',
          'name'        : 'Edit',
          'action'      : 'string:${object_url}/base_edit',
          'condition'     : 'python:not object.hasLockSupport() or object.canToggleLock()',
          'permissions' : (CMFCorePermissions.ModifyPortalContent,),
          'visible'     : 1,
         },
         {
          'id'            : 'lock_unlock',
          'name'          : 'Unlock (Currently locked)',
          'action'        : 'string:${object_url}/unlockExFile',
          'permissions'   : (CMFCorePermissions.View, ),
          'condition'     : 'python:object.isLocked() and object.canToggleLock()',
          'category'      : 'document_actions',
         },
         {
          'id'            : 'unlock_lock',
          'name'          : 'Lock (Currently not locked)',
          'action'        : 'string:${object_url}/lockExFile',
          'permissions'   : (CMFCorePermissions.View, ),
          'condition'     : 'python:not object.isLocked() and object.canToggleLock()',
          'category'      : 'document_actions',
         },
         {
          'id'          : 'metadata',
          'name'        : 'Properties',
          'action'      : 'string:${object_url}/base_metadata',
          'condition'     : 'python:not object.hasLockSupport() or object.canToggleLock()',
          'permissions' : (CMFCorePermissions.ModifyPortalContent,),
          'visible'     : 1,
         },
         { 'id'         : 'references',
           'name'       : 'References',
           'action'     : 'string:${object_url}/reference_graph',
           'condition'  : 'object/archetype_tool/has_graphviz',
           'permissions': (CMFCorePermissions.ModifyPortalContent, CMFCorePermissions.ReviewPortalContent,),
           'visible'    : True,
           },
         {
          'id'          : 'local_roles',
          'name'        : 'Sharing',
          'action'      : 'string:${object_url}/folder_localrole_form',
          'condition'   : 'python:not object.hasLockSupport() or object.canToggleLock()',
          'permissions' : (CMFCorePermissions.ManageProperties,),
          'visible'     : 1,
          },
        )
    
    __implements__ = ( FileContent.__implements__, )

    security = ClassSecurityInfo()
    
    # Init method
    def __init__(self, oid, **kwargs):
        """__init__(self, oid, **kwargs)"""
        
        ExFileLock.__init__(self)
        FileContent.__init__(self, oid, **kwargs)
        
    security.declarePublic('getIcon')
    def getIcon(self, relative_to_portal=0):
        """Calculate the icon using the mime type of the file
        """
        
        field = self.getField('file')
        portal_url = getToolByName( self, 'portal_url' )
        
        # Find the proper icon
        icon = getattr(self, 'ExFile_small.gif')
        m = getattr(field, 'getSmallIcon', None)
        
        if m is not None:
            icon = m(self)
        
        # Return it
        if relative_to_portal:
            return portal_url.getRelativeContentURL(icon)
            
        # Relative to REQUEST['BASEPATH1']
        res = portal_url(relative=1) + '/' + icon.getId()
        while res[:1] == '/':
            res = res[1:]
        return res

    security.declarePrivate('_migrateGetValue')
    def _migrateGetValue(self, name, new_schema=None):
        """Try to get a value from an object using a variety of methods."""
        schema = self.Schema()
        # Migrate pre-AT 1.3 schemas.
        schema = fixSchema(schema)
        # First see if the new field name is managed by the current schema
        field = schema.get(getattr(new_schema.get(name,None),'old_field_name',name), None)
        if field:
            # At very first try to use the BaseUnit itself
            try:
                if IFileField.isImplementedBy(field):
                    return field.getBaseUnit(self)
            except ConflictError:
                raise
            except:
                pass

            # First try the edit accessor
            try:
                editAccessor = field.getEditAccessor(self)
                if editAccessor:
                    return editAccessor()
            except ConflictError:
                raise
            except:
                pass

            # No luck -- now try the accessor
            try:
                accessor = field.getAccessor(self)
                if accessor:
                    return accessor()
            except ConflictError:
                raise
            except:
                pass
            # No luck use standard method to get the value
            return field.get(self)

            # Still no luck -- try to get the value directly
            # this part should be remove because for some fields this will fail
            # if you get the value directly for example for FixPointField
            # stored value is (0,0) but the input value is a string.
            # at this time FixPointField fails if he got a tuple as input value
            # Because of this line value = value.replace(',','.')
            try:
                return self[field.getName()]
            except ConflictError:
                raise
            except:
                pass

        # Nope -- see if the new accessor method is present
        # in the current object.
        if new_schema:
            new_field = new_schema.get(name)
            # Try the new edit accessor
            try:
                editAccessor = new_field.getEditAccessor(self)
                if editAccessor:
                    return editAccessor()
            except ConflictError:
                raise
            except:
                pass

            # Nope -- now try the accessor
            try:
                accessor = new_field.getAccessor(self)
                if accessor:
                    return accessor()
            except ConflictError:
                raise
            except:
                pass

            # Still no luck -- try to get the value directly using the new name
            try:
                return self[new_field.getName()]
            except ConflictError:
                raise
            except:
                pass

        # Nope -- now see if the current object has an attribute
        # with the same name
        # as the new field
        if shasattr(self, name):
            return getattr(self, name)

        raise ValueError, 'name = %s' % (name)
   
def registerPloneExFile(klass):
    # Archetypes registering
    registerType(klass, PROJECTNAME)


registerPloneExFile(PloneExFile)
